package edu.jhu.cs.oose.group3.intelligentbook.model;

import java.util.HashSet;
import java.util.Set;
import javax.persistence.Basic;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;

/**
 * Represents a single contact item (e.g. a phone number).
 * 
 * Stores the item's value as well as information about who can see it. This
 * visibility information is customizable per-contact item
 */
@Entity
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn(name = "citype", discriminatorType = DiscriminatorType.STRING)
public abstract class ContactItem {
	@Id
	@GeneratedValue
	long contactItemId;

	/** The name used to identify the contact item */
	@Basic
	private String name;

	/** The actual string contents of the item */
	@Basic
	private String contents;

	/**
	 * Get the name used to identify the contact item
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Set the name used to identify the contact item
	 * 
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the contents
	 */
	public String getContents() {
		return contents;
	}

	/**
	 * @param contents
	 *            the contents to set
	 */
	public void setContents(String contents) {
		this.contents = contents;
	}

	/** A set of groups that this contact item is visible to */
	@ManyToMany
	private Set<ContactGroup> visibleGroups = new HashSet<ContactGroup>();

	/**
	 * @return the set of groups that this item is visible to
	 */
	/*
	 * public Set<ContactGroup> getVisibleGroups() { return visibleGroups; }
	 */

	/**
	 * Make this item either visible or invisible to the specified group
	 * 
	 * @param group
	 * @param visibility
	 */
	public void setVisibleTo(ContactGroup group, boolean visibility) {
		if (visibleGroups.contains(group)) {
			if (visibility == false)
				visibleGroups.remove(group);
		}
		else if (visibility == true) {
			visibleGroups.add(group);
		}
	}

	/**
	 * @param group
	 * @return whether or not this item is visible to the group
	 */
	public boolean isVisibleTo(ContactGroup group) {
		return visibleGroups.contains(group);
	}

	/**
	 * The item is visible to a contact if and only if the contact is a member
	 * of at least one group in visibleGroups
	 * 
	 * @param contact
	 * @return whether or not this item is visible to the specified contact
	 */
	public boolean isVisibleTo(Contact contact) {
		for (ContactGroup g : contact.getAllGroups())
			if (visibleGroups.contains(g))
				return true;
		return false;
	}
	
	/** Make this item visible to no groups */
	public void resetVisibility() {
		visibleGroups.clear();
	}

	public ContactItem(String contents) {
		this.contents = contents;
		this.name = this.getClass().getSimpleName();
	}

	public ContactItem() {
		this.name = this.getClass().getSimpleName();
	}

}
